// "Depth of Field" demo for Ogre
// Copyright (C) 2006  Christian Lindequist Larsen
//
// This code is in the public domain. You may do whatever you want with it.

#ifdef GL_ES
precision mediump int;
precision highp float;
#endif

// based on "Advanced Depth of Field" by "Thorsten Scheuermann"
#define USE_OGRE_FROM_FUTURE
#include <OgreUnifiedShader.h>

#define NUM_TAPS 12						// number of taps the shader will use

SAMPLER2D(RT, 0);				    // full resolution image
SAMPLER2D(depth, 1);				// full resolution image with depth values
SAMPLER2D(blur,  2);				// downsampled and blurred image

OGRE_UNIFORMS(
uniform vec4 pixelSizeScene;			// pixel size of full resolution image
uniform float dofBlur;
uniform float dofRange;
uniform vec4 cameraProjection;
uniform vec4 cameraParams;
)

// dofParams coefficients:
// x = near blur depth; y = focal plane depth; z = far blur depth
// w = blurriness cutoff constant
//STATIC vec4 dofParams = vec4(0.9991, 0.9985, 0.9975, 1);

float getBlurAmount(float depth)
{
    float f;
    
    depth = cameraProjection.y / (depth - cameraProjection.x);
    float dfocal = (cameraParams.z - cameraParams.x) / (cameraParams.y - cameraParams.x);
    float mindepth = dfocal + dofRange;
    float maxdepth = dfocal - dofRange;
    
    if (depth > dfocal)
    {
        // scale depth value between near blur distance and focal distance
        f = (mindepth - depth) / (mindepth - dfocal);
    }
    else
    {
        // scale depth value between focal distance and far blur distance
        f = (depth - maxdepth) / (dfocal - maxdepth);
    }

    return 1.0 - clamp(f, 0.0, dofBlur);
}

MAIN_PARAMETERS
IN(vec2 oUv0, TEXCOORD0)
MAIN_DECLARATION
{
    vec2 CoC = vec2(5.0, 10.0);	         // max and min circle of confusion (CoC) radius
    vec2 pixelSizeBlur = pixelSizeScene.zw * vec2_splat(4.0); // pixel size of downsampled and blurred image

    vec2 poisson[NUM_TAPS];					     // contains poisson-distributed positions on the unit circle
    poisson[ 0] = vec2( 0.00,  0.00);
    poisson[ 1] = vec2( 0.07, -0.45);
    poisson[ 2] = vec2(-0.15, -0.33);
    poisson[ 3] = vec2( 0.35, -0.32);
    poisson[ 4] = vec2(-0.39, -0.26);
    poisson[ 5] = vec2( 0.10, -0.23);
    poisson[ 6] = vec2( 0.36, -0.12);
    poisson[ 7] = vec2(-0.31, -0.01);
    poisson[ 8] = vec2(-0.38,  0.22);
    poisson[ 9] = vec2( 0.36,  0.23);
    poisson[10] = vec2(-0.13,  0.29);
    poisson[11] = vec2( 0.14,  0.41);

    // Get depth of center tap and convert it into blur radius in pixels
    float centerDepth = texture2D(depth, oUv0).r;
    float centerBlur = getBlurAmount(centerDepth);
    float discRadius = max(0.0, centerBlur * CoC.x - CoC.y);

    vec4 sum = vec4_splat(0.0);
    for (int i = 0; i < NUM_TAPS; ++i)
    {
        // compute texture coordinates
        vec2 coordScene = oUv0 + (pixelSizeScene.zw * poisson[i] * vec2_splat(discRadius));
        vec2 coordBlur = oUv0 + (pixelSizeBlur * poisson[i] * vec2_splat(discRadius));

        // mix low and high res. taps based on tap blurriness
        float blurAmount = getBlurAmount(texture2D(depth, coordScene).r); // put blurriness into [0, 1]
        vec4 tap = mix(texture2D(RT, coordScene), texture2D(blur, coordBlur), blurAmount);

        // accumulate
        sum.rgb += tap.rgb * vec3_splat(blurAmount);
        sum.a += blurAmount;
    }
    
    gl_FragColor = sum / vec4_splat(sum.a);
}
